#include "stdafx.h"
#include "DetectServer.h"


int cvuIsSafeExtension( const char *filename )
{
	char *exts[30] = { ".BMP", ".DIB", ".JPEG", ".JPG", ".JPE",  ".PNG", ".PBM",
					   ".PGM", ".PPM", ".SR",   ".RAS", ".TIFF", ".TIF", ".EXR", ".JP2",
					   ".bmp", ".dib", ".jpeg", ".jpg", ".jpe",  ".png", ".pbm",
					   ".pgm", ".ppm", ".sr",   ".ras", ".tiff", ".tif", ".exr", ".jp2"
	};

	if ( filename[0] == '\0' || filename == NULL ) {
		fprintf(stderr, "[WARNING] filename is null. \n");
		return FALSE;
	}
	const char *ext = strrchr(filename,'.');
	if ( ext == NULL ) {
		fprintf(stderr, "[WARNING] filename has no extension. \n");
		return FALSE;
	}
	bool ext_is_ok = false;
	for (int i=0; i<30; i++) {
		if ( strstr(ext,exts[i]) ) {
			ext_is_ok = true;
			break;
		}
	}
	if ( !ext_is_ok ) {
		fprintf(stderr, "[WARNING] filename has bad extension. \n");
		return FALSE;
	}

	return TRUE;
}

int cvuSafeSaveImage( const char *filename, const CvArr *image, const int *params )
{
	if ( !cvuIsSafeExtension(filename) ) return FALSE;
	FILE *fp = NULL;
	fopen_s( &fp, filename, "w" );
	if ( fp == NULL ) {
		fprintf(stderr,"[WARNING] directory is not found. \n");
		return FALSE;
	}
	fclose(fp);
	return cvSaveImage( filename, image, params );
}


IplImage* cvuSafeLoadImage( const char *filename, int iscolor )
{
	if ( !cvuIsSafeExtension(filename) ) return NULL;
	FILE *fp = NULL;
	fopen_s( &fp, filename,"r");
	if ( fp == NULL ) {
		fprintf(stderr,"[WARNING] file is not found. \n");
		return NULL;
	}
	fclose(fp);
	return cvLoadImage(filename, iscolor);
}


void SendSameNamedData( Network *net, const vector<AR_OBJECT> &marker_info, DATA_ENABLE_LIST enable_list, string &name )
{
	char buf[1048];
	int num_found = 0;

	for (unsigned int i=0; i<marker_info.size(); i++) {
		if ( marker_info[i].name == name ) {
			num_found++;
		}
	}

	sprintf_s( buf, sizeof(buf), "Num= %d\n", num_found );
	net->send_message(buf);

	for (unsigned int i=0; i<marker_info.size(); i++) {
		if ( marker_info[i].name == name ) {
			MakeMarkerInfoString( marker_info[i], enable_list, buf, sizeof(buf) );
			strcat_s( buf, sizeof(buf), "\n" );
			net->send_message(buf);
		}
	}
}


bool SendAllData( Network *net, const vector<AR_OBJECT> &marker_info, DATA_ENABLE_LIST enable_list )
{
	char buf[1024];
	int num = marker_info.size();

	boost::system::error_code ec;
	const int error_value = boost::asio::error::connection_reset;

	sprintf_s( buf, sizeof(buf), "Num= %d\n", num );
	ec = net->send_message(buf);
	if (ec.value()==error_value) {
		return false;
	}

	for ( int j=0; j<num; j++) {
		MakeMarkerInfoString( marker_info[j], enable_list, buf, sizeof(buf) );
		strcat_s( buf, sizeof(buf), "\n");
		ec = net->send_message(buf);
		if (ec.value()==error_value) return false;
	}

	return true;
}


void cvuDrawString( IplImage *image, const char *text, CvPoint pos, CvScalar color, double scale)
{
	CvFont dfont;
	double italicscale  = 0.0f;
    int    thickness;
	
	thickness = 4;
    cvInitFont( &dfont, CV_FONT_HERSHEY_DUPLEX, scale, scale,
				italicscale, thickness, CV_AA);
	cvPutText( image, text, pos, &dfont, CV_RGB(0,0,0) );

	thickness = 1;
	cvInitFont( &dfont, CV_FONT_HERSHEY_DUPLEX, scale, scale,
				italicscale, thickness, CV_AA);
	cvPutText( image, text, pos, &dfont, color );
}

int DrawMarkerLine( IplImage *image, AR_OBJECT &object, CvScalar color,
					int thickness, int line_type, int shift)
{
	// edge line
	CvPoint v[4];
	for (int i=0; i<4; i++) {
		v[i] = cvPoint( (int)(object.vertex[i][0]), (int)(object.vertex[i][1]) );
	}
	for (int i=0; i<4; i++) {
		cvLine( image, v[i], v[(i+1)%4], color, thickness, line_type, shift );
	}

	// direction vector
	CvPoint vec[2];
	for (int i=0; i<2; i++) {
		vec[i].x = (int)(v[i].x - object.px);
		vec[i].y = (int)(v[i].y - object.py);
	}
	CvPoint dvecpos;
	dvecpos.x = (int)( (vec[0].x + vec[1].x) + object.px );
	dvecpos.y = (int)( (vec[0].y + vec[1].y) + object.py );

	CvPoint center = cvPoint( (int)(object.px), (int)(object.py) );
	cvLine( image, center, dvecpos, color, thickness, line_type, shift );

	return 0;
}

void DrawMarkerInfo( IplImage *image, AR_OBJECT &object )
{
	char text[256];
	DrawMarkerLine( image, object, CV_RGB(255,0,0), 1 );

	// coordinate value
	sprintf_s( text, sizeof(text), "(%d,%d)",(int)(object.px),(int)(object.py));
	cvuDrawString( image, text, cvPoint( (int)(object.px), (int)(object.py) ), CV_RGB(255,255,0), 0.5 );

	// name
	cvuDrawString( image, object.name.c_str(), cvPoint( (int)(object.px-10), (int)(object.py + 50) ), CV_RGB(255,255,255), 1.0 );
}

void Detection( IplImage* imgIn, IplImage *imgARTK, int thresh, const vector<PATTERN_DATA> &pattern_list, vector<AR_OBJECT> &detected_markers )
{
	cvCvtColor( imgIn, imgARTK, CV_BGR2BGRA );
	detected_markers.clear();	// important!
	DetectMarker( (ARUint8*)(imgARTK->imageData), thresh, pattern_list, detected_markers );
}


void MakeMarkerInfoString( const AR_OBJECT &object, DATA_ENABLE_LIST elist, char buf[], int bufsize )
{
	bool token = false;
	sprintf_s( buf, bufsize, "" );	// init string

	if (elist.name) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s%s" , buf, object.name.c_str() );
		token = true;
	}
	if (elist.pos2d) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s%d,", buf, (int)(object.px) );
		sprintf_s( buf, bufsize, "%s%d", buf, (int)(object.py) );
		token = true;
	}
	if (elist.dvec) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s%lf,", buf, object.dx );
		sprintf_s( buf, bufsize, "%s%lf" , buf, object.dy );
		token = true;
	}
	if (elist.pos3d) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s%lf,", buf, object.X  );
		sprintf_s( buf, bufsize, "%s%lf,", buf, object.Y  );
		sprintf_s( buf, bufsize, "%s%lf" , buf, object.Z  );
		token = true;
	}
	if (elist.area) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s%d" , buf, object.area );
		token = true;
	}
	if (elist.cf) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s%lf", buf, object.cf );
		token = true;
	}
	if (elist.vertex) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		for (int i=0; i<4; i++) {
			if (i>0) sprintf_s( buf, bufsize, "%s,", buf);
			sprintf_s( buf, bufsize, "%s%lf,", buf, object.vertex[i][0] );
			sprintf_s( buf, bufsize, "%s%lf",  buf, object.vertex[i][2] );
		}
		token = true;
	}
	if (elist.glmatrix) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		for (int i=0; i<16; i++) {
			if (i>0) sprintf_s( buf, bufsize, "%s,", buf);
			sprintf_s( buf, bufsize, "%s%lf", buf, object.gltrans[i] );
		}
		token = true;
	}
}

void MakeDataFormatString( DATA_ENABLE_LIST elist, char buf[], int bufsize )
{
	bool token = false;
	sprintf_s( buf, bufsize, "" );	// init string

	if (elist.name) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s<name>", buf);
		token = true;
	}
	if (elist.pos2d) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s<px>,<py>", buf);
		token = true;
	}
	if (elist.dvec) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s<dx>,<dy>", buf);
		token = true;
	}
	if (elist.pos3d) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s<X>,<Y>,<Z>", buf);
		token = true;
	}
	if (elist.area) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s<area>" , buf);
		token = true;
	}
	if (elist.cf) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		sprintf_s( buf, bufsize, "%s<cf>", buf);
		token = true;
	}
	if (elist.vertex) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		for (int i=0; i<4; i++) {
			if (i>0) sprintf_s( buf, bufsize, "%s,",buf);
			sprintf_s( buf, bufsize, "%s<vx%d>,<vy%d>", buf,i,i);
		}
		token = true;
	}
	if (elist.glmatrix) {
		if (token) sprintf_s( buf, bufsize, "%s,", buf);
		for (int i=0; i<16; i++) {
			if (i>0) sprintf_s( buf, bufsize, "%s,", buf);
			//sprintf_s( buf, bufsize, "%s<m%d>", buf, i );
			int a = i%4 + 1;
			int b = (int)(i/4) + 1;
			sprintf_s( buf, bufsize, "%s<m%d%d>", buf, a, b );			
		}
		token = true;
	}
}

bool InBoundingBox( const ARMarkerInfo *marker, int x, int y )
{
	int max_x=-1, min_x=10000, max_y=-1, min_y=10000;

	for (int i=0; i<4; i++) {
		if ( max_x < marker->vertex[i][0] ) max_x = (int)(marker->vertex[i][0]);
		if ( max_y < marker->vertex[i][1] ) max_y = (int)(marker->vertex[i][1]);
		if ( min_x > marker->vertex[i][0] ) min_x = (int)(marker->vertex[i][0]);
		if ( min_y > marker->vertex[i][1] ) min_y = (int)(marker->vertex[i][1]);
	}
	return ( min_x <= x && min_y <= y && x <= max_x && y < max_y );
}



int DrawMarkerLine2( IplImage *image, const ARMarkerInfo *marker,
				    CvScalar color, int thickness, int line_type, int shift )
{
	// edge line
	CvPoint v[4];
	v[0] = cvPoint( (int)(marker->vertex[2][0]), (int)(marker->vertex[2][1]));
	v[1] = cvPoint( (int)(marker->vertex[3][0]), (int)(marker->vertex[3][1]));
	v[2] = cvPoint( (int)(marker->vertex[0][0]), (int)(marker->vertex[0][1]));
	v[3] = cvPoint( (int)(marker->vertex[1][0]), (int)(marker->vertex[1][1]));
	for (int i=0; i<4; i++) {
		cvLine( image, v[i], v[(i+1)%4], color, thickness, line_type, shift );
	}
	
	// direction vector
	CvPoint vec[2];

	for (int i=0; i<2; i++) {
		vec[i].x = (int)(v[i].x - marker->pos[0]);
		vec[i].y = (int)(v[i].y - marker->pos[1]);
	}
	CvPoint dvecpos;
	dvecpos.x = (vec[0].x + vec[1].x) + (int)(marker->pos[0]);
	dvecpos.y = (vec[0].y + vec[1].y) + (int)(marker->pos[1]);

	CvPoint center = cvPoint( (int)(marker->pos[0]), (int)(marker->pos[1]) );
	cvLine( image, center, dvecpos, color, thickness, line_type, shift );

	return 0;
}

void ClipImage( const IplImage *src, IplImage *dst, const ARMarkerInfo *marker )
{
	CvMat src_point;
	double a[] = {
		marker->vertex[2][0], marker->vertex[2][1],
		marker->vertex[3][0], marker->vertex[3][1],
		marker->vertex[0][0], marker->vertex[0][1],
		marker->vertex[1][0], marker->vertex[1][1]
	};
	src_point = cvMat( 4, 2, CV_64FC1, a );


	CvMat dst_point;
	double b[] = {
		0, 0,
		dst->width - 1, 0,
		dst->width - 1, dst->height - 1,
		0, dst->height - 1
	};

	dst_point = cvMat( 4, 2, CV_64FC1, b );
	CvMat *h = cvCreateMat( 3, 3, CV_64FC1 );

	cvFindHomography( &src_point, &dst_point, h );
	cvWarpPerspective( src, dst, h );
}
