// QPServerDlg.cpp
//

// auto config of threashold
// 2D/3D calibration
// multi marker
// high speed processing

// Number of markers is limited by AR_PATT_NUM_MAX in config.h (in ARToolKit)
// Image size is limited by HARDCODED_BUFFER_WIDTH and HARDCODED_BUFFER_HEIGHT in arLabeling.c (in ARToolKit)

#include "stdafx.h"
#include "QPServer.h"
#include "QPServerDlg.h"
#include "DetectServer.h"
#include "BitmapUtil.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define IDM_DELETE_ITEM 1500


//uގO[oϐ`ĂEEEɂv

static boost::mutex	g_mtServerEnable;
static bool g_serverEnable = false;
static boost::mutex g_mtMarkerInfo;
static vector<AR_OBJECT> g_markerInfo;
static bool g_auto_dataflow;
static DATA_ENABLE_LIST g_enable_list;
static IplImage* g_imgCapture = NULL;	// do not release
static bool g_regist_patt = false;
static string g_regist_file;

typedef struct {
	bool L;
	bool R;
	bool M;
	int x;
	int y;
} MOUSE_STATE;

static MOUSE_STATE g_mouse;

#define PATH_CONFIG_FILE		"./Data/config.txt"
#define PATH_PATTERN_LIST_FILE	"./Data/pattern_list.txt"
#define PATH_CAMERA_PARAM_FILE	"./Data/camera_para.dat"
#define PATH_NOPATTERN_IMAGE	"./no_pattern.bmp"
#define PATH_DEFAULT_IMAGE		"./Data/000.jpg"


bool IsAccessFromMe(Network *net)
{
	string ip_address = net->get_ip_address();
	return ( ip_address == "127.0.0.1" || ip_address == GetLocalIPAddress() );
}


string CString2string( const CString &src )
{
	char *cbuf = new char[src.GetLength()+1];
	int k;
	for ( k=0; k< src.GetLength(); k++) {
		cbuf[k] = (char)(src[k]);
	}
	cbuf[k]='\0';
	string dst = cbuf;
	delete[] cbuf;
	return dst;
}

int DetectFromImage(string filename, vector<AR_OBJECT> &marker_info)
{
	vector<PATTERN_DATA> patt_list;
	LoadPatternList( PATH_PATTERN_LIST_FILE, patt_list );

	IplImage* imgFile = cvuSafeLoadImage(filename.c_str());
	if ( imgFile == NULL ) return -1;
	IplImage* imgARTK = cvCreateImage(cvGetSize(imgFile),IPL_DEPTH_8U, 4);
	if ( imgARTK == NULL ) return -1;

	Detection( imgFile, imgARTK, 100, patt_list, marker_info);
	cvReleaseImage(&imgFile);
	cvReleaseImage(&imgARTK);

	return 0;
}


int InterpretCommand( string line, DATA_ENABLE_LIST enable_list, Network *net)
{
	stringstream ss(line);
	string cmd;
	ss >> cmd;

	if ( cmd=="help" ) {
		net->send_message("<<< COMMAND LIST >>>\n");
		net->send_message("all        Get all marker information.\n");
		net->send_message("get <name> Get marker information having specified name.\n");			
		net->send_message("help       Show command list.\n");
		net->send_message("exit       Dissconnect this connection.\n");
		net->send_message("quit       Dissconnect this connection.\n");
		net->send_message("saveiamge <path> Save image. Do not include space in <path>.\n");
		net->send_message("register <path> Register marker from image involving a marker.\n");
		net->send_message("allfromimage <path> Get all marker from image.\n");
		net->send_message("getfromimage <path> <name> Get specified marker from image.\n");
		net->send_message("\n");
	}
	else if ( cmd=="get" ) {
		string name;
		ss >> name;
		g_mtMarkerInfo.lock();
			SendSameNamedData( net, g_markerInfo, enable_list, name );
		g_mtMarkerInfo.unlock();
	}
	else if ( cmd=="all" ) {
		g_mtMarkerInfo.lock();
			SendAllData( net, g_markerInfo, enable_list );
		g_mtMarkerInfo.unlock();
	}
	else if ( cmd=="allfromimage" ) {	// Too large file is not better
		string filename;
		ss >> filename;
		vector<AR_OBJECT> marker_info;
		DetectFromImage(filename,marker_info);
		SendAllData( net, marker_info, enable_list );
	}
	else if ( cmd=="getfromimage" ) {	// Too large file is not better
		string filename;
		string name;
		ss >> filename >> name;
		vector<AR_OBJECT> marker_info;
		DetectFromImage(filename,marker_info);
		SendSameNamedData( net, marker_info, enable_list, name );
	}
	else if ( cmd=="saveimage" ) {
		if ( !(IsAccessFromMe(net)) ) {	// only this computer
			net->send_message("*error : The command is not accepted.\n");
		}
		else {
			string filename;
			ss >> filename;
			g_mtMarkerInfo.lock();
				int ret = cvuSafeSaveImage( filename.c_str(), g_imgCapture );
			g_mtMarkerInfo.unlock();
			if (ret==TRUE) net->send_message("*ok\n");
			else net->send_message("*error\n");
		}
	}
	else if ( cmd=="register" ) {
		if ( !(IsAccessFromMe(net)) ) {	// only this computer
			net->send_message("*error : The command is not accepted.\n");
		}
		else {
			// signal to main thread
			ss >> g_regist_file;
			g_regist_patt = true;
			while (g_regist_patt) {
				Sleep(10);
			}
			// ́CłɂԍԂƂD
			vector<AR_OBJECT> marker_info;
			DetectFromImage( g_regist_file.c_str(), marker_info );
			if ( marker_info.size()>0) {
				net->send_message( marker_info[0].name + "\n" );
			}else {
				net->send_message( "<NONE>\n" );
			}
		}
	}
	else if ( cmd=="quit" || cmd=="exit" ) {
		net->send_message("*ok\n");
		return -1;
	}
	else {
		net->send_message("*error\n");
	}
	return 0;
}


void Server(Network &net)
{
	TRACE("Server() start\n");

	// Maybe, these global variables are safety before loop
	bool is_auto_mode = g_auto_dataflow;
	DATA_ENABLE_LIST enable_list = g_enable_list;
	//-------------------------------------------------

	int ms;
	LARGE_INTEGER freq, time_curr, time_prev;
	QueryPerformanceFrequency(&freq);
	QueryPerformanceCounter(&time_prev);

	net.send_message("**********************\n");
	net.send_message(" Welcome to QPServer! \n");
	net.send_message("**********************\n");
		
	while (1) {
		// to make constant communication rate -----------------------------------------
		for (;;) {
			QueryPerformanceCounter(&time_curr);
			ms = (int)((time_curr.QuadPart - time_prev.QuadPart)*1000 / freq.QuadPart);
			if (ms>=33) break;
		}
		time_prev = time_curr;

		// Check pushing stop button --------------------------------------------
		g_mtServerEnable.lock();
		if ( !g_serverEnable ) {
			g_mtServerEnable.unlock();
			break;
		}
		g_mtServerEnable.unlock();

		// Send result ----------------------------------------------------------

		// AUTOMATIC MODE
		if ( is_auto_mode ) {
			g_mtMarkerInfo.lock();
				bool success = SendAllData( &net, g_markerInfo, enable_list );
			g_mtMarkerInfo.unlock();
			if ( !success ) break;
		}
		// REQUEST MODE
		else {
			string msg = net.read_message();
			if (msg.size() == 0) break;
			stringstream ss(msg);
			string line;
			int ret = 0;
			while ( getline(ss,line) ) {
				ret = InterpretCommand(line,enable_list,&net);
				if (ret==-1) break;
			}
			if (ret==-1) break;
		}
	}
	TRACE("Server() end\n");
}



void on_mouse (int event, int x, int y, int flags, void *param = NULL)
{
	g_mouse.x = x;
	g_mouse.y = y;

	switch (event) {
		case CV_EVENT_LBUTTONDOWN:
			g_mouse.L = true;
			break;
		case CV_EVENT_LBUTTONUP:
			g_mouse.L = false;
			break;
		case CV_EVENT_RBUTTONDOWN:
			g_mouse.R = true;
			break;
		case CV_EVENT_RBUTTONUP:
			g_mouse.R = false;
			break;
		case CV_EVENT_MBUTTONDOWN:
			g_mouse.M = true;
			break;
		case CV_EVENT_MBUTTONUP:
			g_mouse.M = false;
			break;
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// AvP[Ṽo[WɎg CAboutDlg _CAO

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// _CAO f[^
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV T|[g

// 
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// CQPServerDlg _CAO
CQPServerDlg::CQPServerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CQPServerDlg::IDD, pParent)
	, m_edit_port(0)
	, m_combo_camera_id(0)
	, m_combo_image_size(0)
	, m_radio_dataflow(0)
	, m_check_name(FALSE)
	, m_check_pos2d(FALSE)
	, m_check_dvec(FALSE)
	, m_check_pos3d(FALSE)
	, m_check_glmatrix(FALSE)
	, m_check_vertex(FALSE)
	, m_check_area(FALSE)
	, m_check_cf(FALSE)
	, m_edit_data_format(_T(""))
	, m_edit_patt_width(0)
	, m_edit_patt_name(_T(""))
	, m_check_preview(FALSE)
	, m_edit_output(_T(""))
	, m_edit_IP_address(_T(""))
	, m_check_auto_start(FALSE)
	, m_combo_fps(0)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CQPServerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BUTTON_START, m_button_start);
	DDX_Control(pDX, IDC_BUTTON_STOP, m_button_stop);
	DDX_Text(pDX, IDC_EDIT_PORT, m_edit_port);
	DDX_Control(pDX, IDC_EDIT_PORT, m_cntl_edit_port);
	DDX_CBIndex(pDX, IDC_COMBO_CAMERA_ID, m_combo_camera_id);
	DDX_Control(pDX, IDC_COMBO_CAMERA_ID, m_cntl_combo_camera_id);
	DDX_CBIndex(pDX, IDC_COMBO_IMAGE_SIZE, m_combo_image_size);
	DDX_Control(pDX, IDC_COMBO_IMAGE_SIZE, m_cntl_combo_image_size);
	DDX_Radio(pDX, IDC_RADIO_DATAFLOW1, m_radio_dataflow);
	DDX_Check(pDX, IDC_CHECK_NAME, m_check_name);
	DDX_Check(pDX, IDC_CHECK_POS2D, m_check_pos2d);
	DDX_Check(pDX, IDC_CHECK_DVEC, m_check_dvec);
	DDX_Check(pDX, IDC_CHECK_POS3D, m_check_pos3d);
	DDX_Check(pDX, IDC_CHECK_GLMATRIX, m_check_glmatrix);
	DDX_Check(pDX, IDC_CHECK_VERTEX, m_check_vertex);
	DDX_Check(pDX, IDC_CHECK_AREA, m_check_area);
	DDX_Check(pDX, IDC_CHECK_CF, m_check_cf);
	DDX_Control(pDX, IDC_STATIC_FORMAT, m_cntl_data_format);
	DDX_Control(pDX, IDC_CHECK_NAME, m_cntl_check_name);
	DDX_Control(pDX, IDC_CHECK_POS2D, m_cntl_check_pos2d);
	DDX_Control(pDX, IDC_CHECK_DVEC, m_cntl_check_dvec);
	DDX_Control(pDX, IDC_CHECK_POS3D, m_cntl_check_pos3d);
	DDX_Control(pDX, IDC_CHECK_GLMATRIX, m_cntl_check_glmatrix);
	DDX_Control(pDX, IDC_CHECK_VERTEX, m_cntl_check_vertex);
	DDX_Control(pDX, IDC_CHECK_AREA, m_cntl_check_area);
	DDX_Control(pDX, IDC_CHECK_CF, m_cntl_check_cf);
	DDX_Text(pDX, IDC_EDIT_DATA_FORMAT, m_edit_data_format);
	DDX_Control(pDX, IDC_RADIO_DATAFLOW1, m_cntl_radio_dataflow1);
	DDX_Control(pDX, IDC_RADIO_DATAFLOW2, m_cntl_radio_dataflow2);
	DDX_Control(pDX, IDC_LIST_PATTERN, m_cntl_list_pattern);
	DDX_Control(pDX, IDC_BUTTON_START_PATTERN_CAPTURE, m_cntl_button_patt_capture);
	DDX_Control(pDX, IDC_EDIT_PATTERN_NAME, m_cntl_edit_patt_name);
	DDX_Control(pDX, IDC_EDIT_PATTERN_WIDTH, m_cntl_edit_patt_width);
	DDX_Text(pDX, IDC_EDIT_PATTERN_WIDTH, m_edit_patt_width);
	DDX_Text(pDX, IDC_EDIT_PATTERN_NAME, m_edit_patt_name);
	DDX_Check(pDX, IDC_CHECK_PREVIEW, m_check_preview);
	DDX_Control(pDX, IDC_CHECK_PREVIEW, m_cntl_check_preview);
	DDX_Control(pDX, IDC_EDIT_OUTPUT, m_cntl_edit_output);
	DDX_Text(pDX, IDC_EDIT_OUTPUT, m_edit_output);
	DDX_Text(pDX, IDC_EDIT_IP_ADDRESS, m_edit_IP_address);
	DDX_Check(pDX, IDC_CHECK_AUTO_START, m_check_auto_start);
	DDX_Control(pDX, IDC_COMBO_FPS, m_cntl_combo_fps);
	DDX_CBIndex(pDX, IDC_COMBO_FPS, m_combo_fps);
	DDX_Control(pDX, IDC_BUTTON_STOP_PATTERN_CAPTURE, m_cntl_button_stop_patt_cap);
}

BEGIN_MESSAGE_MAP(CQPServerDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDOK, &CQPServerDlg::OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, &CQPServerDlg::OnBnClickedCancel)
	ON_BN_CLICKED(IDC_BUTTON_STOP, &CQPServerDlg::OnBnClickedButtonStop)
	ON_BN_CLICKED(IDC_BUTTON_START, &CQPServerDlg::OnBnClickedButtonStart)
	ON_BN_CLICKED(IDC_CHECK_NAME, &CQPServerDlg::OnBnClickedCheckName)
	ON_BN_CLICKED(IDC_CHECK_POS2D, &CQPServerDlg::OnBnClickedCheckPos2d)
	ON_BN_CLICKED(IDC_CHECK_DVEC, &CQPServerDlg::OnBnClickedCheckDvec)
	ON_BN_CLICKED(IDC_CHECK_POS3D, &CQPServerDlg::OnBnClickedCheckPos3d)
	ON_BN_CLICKED(IDC_CHECK_AREA, &CQPServerDlg::OnBnClickedCheckArea)
	ON_BN_CLICKED(IDC_CHECK_CF, &CQPServerDlg::OnBnClickedCheckCf)
	ON_BN_CLICKED(IDC_CHECK_VERTEX, &CQPServerDlg::OnBnClickedCheckVertex)
	ON_BN_CLICKED(IDC_CHECK_GLMATRIX, &CQPServerDlg::OnBnClickedCheckGlmatrix)
	ON_NOTIFY(NM_RCLICK, IDC_LIST_PATTERN, &CQPServerDlg::OnNMRClickListPattern)
	ON_BN_CLICKED(IDC_BUTTON_START_PATTERN_CAPTURE, &CQPServerDlg::OnBnClickedButtonStartPatternCapture)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST_PATTERN, &CQPServerDlg::OnLvnItemchangedListPattern)
	ON_BN_CLICKED(IDM_DELETE_ITEM,&CQPServerDlg::DeleteItemFromList)
	ON_EN_CHANGE(IDC_EDIT_PORT, &CQPServerDlg::OnEnChangeEditPort)
	ON_BN_CLICKED(IDC_CHECK_AUTO_START, &CQPServerDlg::OnBnClickedCheckAutoStart)
	ON_BN_CLICKED(IDC_CHECK_PREVIEW, &CQPServerDlg::OnBnClickedCheckPreview)
	ON_BN_CLICKED(IDC_RADIO_DATAFLOW1, &CQPServerDlg::OnBnClickedRadioDataflow1)
	ON_BN_CLICKED(IDC_RADIO_DATAFLOW2, &CQPServerDlg::OnBnClickedRadioDataflow2)
	ON_CBN_SELCHANGE(IDC_COMBO_CAMERA_ID, &CQPServerDlg::OnCbnSelchangeComboCameraId)
	ON_CBN_SELCHANGE(IDC_COMBO_IMAGE_SIZE, &CQPServerDlg::OnCbnSelchangeComboImageSize)
	ON_EN_CHANGE(IDC_EDIT_PATTERN_NAME, &CQPServerDlg::OnEnChangeEditPatternName)
	ON_EN_CHANGE(IDC_EDIT_PATTERN_WIDTH, &CQPServerDlg::OnEnChangeEditPatternWidth)
	ON_CBN_SELCHANGE(IDC_COMBO_FPS, &CQPServerDlg::OnCbnSelchangeComboFps)
	ON_BN_CLICKED(IDC_BUTTON_STOP_PATTERN_CAPTURE, &CQPServerDlg::OnBnClickedButtonStopPatternCapture)
END_MESSAGE_MAP()


// CQPServerDlg bZ[W nh

BOOL CQPServerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// "o[W..." j[VXe j[ɒǉ܂B

	// IDM_ABOUTBOX ́AVXe R}h͈͓̔ɂȂ΂Ȃ܂B
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}
	// ̃_CAÕACRݒ肵܂BAvP[ṼC EBhE_CAOłȂꍇA
	//  Framework ́A̐ݒIɍs܂B
	SetIcon(m_hIcon, TRUE);		// Large Icon Config
//	SetIcon(m_hIcon, FALSE);	// Small Icon Config
	SetIcon(m_hIcon, TRUE);		// Small Icon Config

	// TODO: Init here ////////////////////////////////////////////////////////////
	
	// My IP address
	m_edit_IP_address = GetLocalIPAddress().c_str();

	// load config
	LoadConfigFile( PATH_CONFIG_FILE );

	// camera ID
	for (int i=0; i<m_max_camera_num; i++) {
		WCHAR  wcbuf[256];
		wsprintf(wcbuf,TEXT("%d"),i);
		m_cntl_combo_camera_id.InsertString(i,wcbuf);
	}
	UpdateData(false);

	// Image size
	for (unsigned int i=0; i<m_size_list.size(); i++) {
		WCHAR  wcbuf[256];
		wsprintf(wcbuf,TEXT("%dx%d"),m_size_list[i].width,m_size_list[i].height);
		m_cntl_combo_image_size.InsertString(i,wcbuf);
	}
	UpdateData(false);

	// fps
	for (unsigned int i=0; i<m_fps_list.size(); i++) {
        CString str;
		str.Format(_T("%.1lf"), m_fps_list[i]);
		m_cntl_combo_fps.InsertString(i,str);
	}
	UpdateData(false);

	// Load Pettern info
	vector<PATTERN_DATA> pattList;
	LoadPatternList( PATH_PATTERN_LIST_FILE, pattList );
	int err = 0;
	if (!err) err = InitList();
	for (unsigned int i=0; i<pattList.size(); i++) {
		if (!err) err = InsertItemToList(pattList[i]);
	}
	UpdateData(false);

	MakeDataFormat();

	m_imgPatt = NULL;
	if ( m_cntl_list_pattern.GetItemCount()==0 ) {
		m_imgPatt = cvuSafeLoadImage( PATH_NOPATTERN_IMAGE );
	} else {
		m_imgPatt = cvuSafeLoadImage( PATH_DEFAULT_IMAGE );
	}
	ShowImage( IDC_PICT_PATT, m_imgPatt );
	m_cntl_list_pattern.SetItemState( 0, LVIS_SELECTED|LVIS_FOCUSED,LVIS_SELECTED|LVIS_FOCUSED );

	m_pThread = NULL;
	m_pPattCapThread = NULL;
	m_pKillThread = NULL;
	m_loopEnable = false;
	m_PattCapLoopEnable = false;

	if ( m_check_auto_start ) {
		OnBnClickedButtonStart();
	}

	return TRUE;
}

void CQPServerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// _CAOɍŏ{^ǉꍇAACR`悷邽߂
//  ̃R[hKvłBhLg/r[ fg MFC AvP[V̏ꍇA
//  ́AFramework ɂĎIɐݒ肳܂B

void CQPServerDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // `̃foCX ReLXg

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// NCAg̎lp`̈̒
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// ACR̕`
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		ShowImage( IDC_PICT_PATT, m_imgPatt );
		CDialog::OnPaint();
	}
}

// [U[ŏEBhEhbOĂƂɕ\J[\擾邽߂ɁA
//  VXe̊֐Ăяo܂B
HCURSOR CQPServerDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CQPServerDlg::OnBnClickedOk()
{
	// TODO: ɃRg[ʒmnh R[hǉ܂B
	OnOK();
}


BOOL CQPServerDlg::PreTranslateMessage(MSG* pMsg)
{
    switch(pMsg->message){
        case WM_KEYDOWN:
            switch(pMsg->wParam){
                case VK_RETURN:return TRUE;
                case VK_ESCAPE:return TRUE;
            }
    }
    return CDialog::PreTranslateMessage(pMsg);
}


void CQPServerDlg::ShowImage( int nID, IplImage *image )
{
	if (image==NULL) return;

	CWnd*	pWnd = GetDlgItem(nID);
	HDC		hDC = pWnd->GetDC()->GetSafeHdc();
	RECT	rect;
	pWnd->GetClientRect(&rect);

	int SrcY = 0;
	int ViewWidth = rect.right;
	int ViewHeight = rect.bottom;
	int SrcWidth = image->width;
	int SrcHeight = image->height;

	if (ViewWidth <= SrcWidth){
		SrcWidth = ViewWidth;
	}else{
		ViewWidth = SrcWidth;
	}

	BITMAPINFO bmi;
	MakeBitmapInfo24(image->width,image->height,bmi);

	if (ViewHeight <= SrcHeight){
		SrcY = SrcHeight - ViewHeight;
		SrcHeight = ViewHeight;
	}else{
		ViewHeight = SrcHeight;
	}

	StretchDIBits(	hDC,
					0, ViewHeight,
					ViewWidth, -ViewHeight,
					0,0,
					SrcWidth, SrcHeight,
					image->imageData,
					&bmi,
					DIB_RGB_COLORS,SRCCOPY
				);

	::ReleaseDC(m_hWnd,hDC);	// important!!
}


int CQPServerDlg::LoadConfigFile( const char *filename )
{
	ifstream fin(filename);
	if (!fin) {
		cout << filename << " can not open.\n";
		return -1;
	}
	string word;
	while (fin >> word) {
		if (word=="[Port]") {
			fin >> m_edit_port;
		}
		else if (word=="[MaxCameraNum]") {
			fin >> m_max_camera_num;
		}
		else if (word=="[CameraSelected]") {
			fin >> m_combo_camera_id;
		}
		else if (word=="[ImageSizeList]") {
			int width,height;
			fin >> width >> height;
			m_size_list.push_back(cvSize(width,height));
		}
		else if (word=="[ImageSizeSelected]") {
			fin >> m_combo_image_size;
		}
		else if (word=="[FPS]") {
			double fps;
			fin >> fps;
			m_fps_list.push_back(fps);
		}
		else if (word=="[FPSSelected]") {
			fin >> m_combo_fps;
		}
		else if (word=="[DataFlow]") {
			fin >> m_radio_dataflow;
		}
		else if (word=="[CheckName]") {
			fin >> m_check_name;
		}
		else if (word=="[CheckPos2d]") {
			fin >> m_check_pos2d;
		}
		else if (word=="[CheckDvec]") {
			fin >> m_check_dvec;
		}
		else if (word=="[CheckPos3d]") {
			fin >> m_check_pos3d;
		}
		else if (word=="[CheckCf]") {
			fin >> m_check_cf;
		}
		else if (word=="[CheckArea]") {
			fin >> m_check_area;
		}
		else if (word=="[CheckVertex]") {
			fin >> m_check_vertex;
		}
		else if (word=="[CheckGLmatrix]") {
			fin >> m_check_glmatrix;
		}
		else if (word=="[CheckPreview]") {
			fin >> m_check_preview;
		}
		else if (word=="[CheckAutoStart]") {
			fin >> m_check_auto_start;
		}
		else if (word=="[DefaultPatternWidth]") {
			fin >> m_default_patt_width;
		}
	}
	UpdateData(false);
	fin.close();
	return 0;
}

int CQPServerDlg::SaveConfigFile( const char * filename = PATH_CONFIG_FILE )
{
	TRACE("in SaveConfigFile\n");
	UpdateData(true);

	ofstream fout(filename);
	if (!fout) {
		return -1;
	}

	fout << "[Port]                " << m_edit_port << "\n";
	fout << "[MaxCameraNum]        " << m_max_camera_num << "\n";
	fout << "[CameraSelected]      " << m_combo_camera_id << "\n";
	for (unsigned int i=0; i<m_size_list.size(); i++) {
		fout << "[ImageSizeList]       " << m_size_list[i].width << " " << m_size_list[i].height << "\n";
	}
	fout << "[ImageSizeSelected]   " << m_combo_image_size << "\n";
	for (unsigned int i=0; i<m_fps_list.size(); i++) {
		fout << "[FPS]                 " << m_fps_list[i] << "\n";
	}
	fout << "[FPSSelected]         " << m_combo_fps << "\n";
	fout << "[DataFlow]            " << m_radio_dataflow << "\n";
	fout << "[CheckName]           " << m_check_name << "\n";
	fout << "[CheckPos2d]          " << m_check_pos2d << "\n";
	fout << "[CheckDvec]           " << m_check_dvec << "\n";
	fout << "[CheckPos3d]          " << m_check_pos3d << "\n";
	fout << "[CheckCf]             " << m_check_cf << "\n";
	fout << "[CheckArea]           " << m_check_area << "\n";
	fout << "[CheckVertex]         " << m_check_vertex << "\n";
	fout << "[CheckGLmatrix]       " << m_check_glmatrix << "\n";
	fout << "[CheckPreview]        " << m_check_preview << "\n";
	fout << "[CheckAutoStart]      " << m_check_auto_start << "\n";
	fout << "[DefaultPatternWidth] " << m_default_patt_width << "\n";

	fout.close();
	return 0;
}


// Lock UI for main thread
void CQPServerDlg::LockInterface()
{
	m_button_start.EnableWindow(false);
	m_cntl_button_patt_capture.EnableWindow(false);
	m_cntl_button_stop_patt_cap.EnableWindow(false);

	m_cntl_combo_camera_id.EnableWindow(false);
	m_cntl_combo_fps.EnableWindow(false);
	m_cntl_combo_image_size.EnableWindow(false);

	m_cntl_radio_dataflow1.EnableWindow(false);
	m_cntl_radio_dataflow2.EnableWindow(false);

	m_cntl_check_name.EnableWindow(false);
	m_cntl_check_pos2d.EnableWindow(false);
	m_cntl_check_dvec.EnableWindow(false);
	m_cntl_check_pos3d.EnableWindow(false);
	m_cntl_check_glmatrix.EnableWindow(false);
	m_cntl_check_cf.EnableWindow(false);
	m_cntl_check_area.EnableWindow(false);
	m_cntl_check_vertex.EnableWindow(false);

	m_cntl_edit_port.SetReadOnly(true);
	m_cntl_edit_patt_width.SetReadOnly(true);
	m_cntl_edit_patt_name.SetReadOnly(true);
}

// Unlock UI for main thread
void CQPServerDlg::UnlockInterface()
{
	m_button_start.EnableWindow(true);
	m_cntl_button_patt_capture.EnableWindow(true);
	m_cntl_button_stop_patt_cap.EnableWindow(true);

	m_cntl_edit_port.EnableWindow(true);
	m_cntl_combo_camera_id.EnableWindow(true);
	m_cntl_combo_fps.EnableWindow(true);
	m_cntl_combo_image_size.EnableWindow(true);
	m_cntl_data_format.EnableWindow(true);
	m_cntl_radio_dataflow1.EnableWindow(true);
	m_cntl_radio_dataflow2.EnableWindow(true);

	m_cntl_check_name.EnableWindow(true);
	m_cntl_check_pos2d.EnableWindow(true);
	m_cntl_check_dvec.EnableWindow(true);
	m_cntl_check_pos3d.EnableWindow(true);
	m_cntl_check_glmatrix.EnableWindow(true);
	m_cntl_check_cf.EnableWindow(true);
	m_cntl_check_area.EnableWindow(true);
	m_cntl_check_vertex.EnableWindow(true);

	m_cntl_edit_port.SetReadOnly(false);
	m_cntl_edit_patt_width.SetReadOnly(false);
	m_cntl_edit_patt_name.SetReadOnly(false);
}

// StopButton
void CQPServerDlg::OnBnClickedButtonStop()
{
	m_loopEnable = false;
}

// Start Button
void CQPServerDlg::OnBnClickedButtonStart()
{
	// Get settings from GUI
	UpdateData(true);

	// CameraSetup
	int ret = m_capture.Init(m_combo_camera_id);
	if ( ret == -1 ) {
		MessageBox(TEXT("Camera could not be connected.\r\nAnother software probably uses it."), TEXT("Error"), MB_ICONERROR | MB_OK);
		return;
	}
	m_capture.SetConfig( m_size_list[m_combo_image_size].width, m_size_list[m_combo_image_size].height, (int)(m_fps_list[m_combo_fps]) );

	// Enable list from checkbox
	g_enable_list.name  = m_check_name;
	g_enable_list.pos2d = m_check_pos2d;
	g_enable_list.dvec  = m_check_dvec;
	g_enable_list.pos3d = m_check_pos3d;
	g_enable_list.glmatrix = m_check_glmatrix;
	g_enable_list.cf       = m_check_cf;
	g_enable_list.area     = m_check_area;
	g_enable_list.vertex   = m_check_vertex;

	// data flow
	g_auto_dataflow = (m_radio_dataflow==0);

	// Pattern list
	PATTERN_DATA patt;
	m_pattern_list.clear();

	for (int i=0; i<m_cntl_list_pattern.GetItemCount(); i++) {
		patt.id       = i;
		patt.enable   = (m_cntl_list_pattern.GetCheck(i) ? true : false);
		patt.name     = CString2string( m_cntl_list_pattern.GetItemText(i, 0) );
		patt.width    = _tstof(m_cntl_list_pattern.GetItemText(i, 1));
		char filename[256];
		sprintf_s( filename, sizeof(filename), "Data/%03d.patt",i); 
		patt.filename = filename;
		m_pattern_list.push_back(patt);
	}

	LockInterface();
	// error check is necessary
	m_pThread = AfxBeginThread(Thread,this);
}


int CQPServerDlg::SavePatternList(const char *filename = PATH_PATTERN_LIST_FILE )
{
	TRACE("in SavePatternList\n");
//	UpdateData(true);	// do not do this here

	ofstream fout(filename);
	if (!fout) {
		return -1;
	}

	vector<PATTERN_DATA> pattList;
	PATTERN_DATA patt;
	pattList.clear();
	for (int i=0; i<m_cntl_list_pattern.GetItemCount(); i++) {
		patt.enable = (m_cntl_list_pattern.GetCheck(i) ? true : false);
		patt.id       = i;
		patt.name     = CString2string( m_cntl_list_pattern.GetItemText(i, 0) );
		patt.width    = _tstof(m_cntl_list_pattern.GetItemText(i, 1));
		char filename[256];
		sprintf_s( filename, sizeof(filename), "Data/%03d.patt",i); 
		patt.filename = filename;
		pattList.push_back(patt);
	}

	for (unsigned int i=0; i<pattList.size(); i++) {
		fout << "[Pattern]" << "\t" 
			 << pattList[i].name << "\t" 
			 << pattList[i].filename << "\t"
			 << pattList[i].width << "\t"
			 << (pattList[i].enable? "use" : "not_use") << "\n";
	}

	fout.close();

	return 0;
}

void CQPServerDlg::OnBnClickedButtonStartPatternCapture()
{
	// Get settings from GUI
	UpdateData();

	// CameraSetup
	int ret = m_capture.Init(m_combo_camera_id);
	if ( ret == -1 ) {
		MessageBox(TEXT("Camera could not be connected.\r\nAnother software probably uses it."), TEXT("Error"), MB_ICONERROR | MB_OK);
		return;
	}
	m_capture.SetConfig( m_size_list[m_combo_image_size].width, m_size_list[m_combo_image_size].height, (int)(m_fps_list[m_combo_fps]) );

	m_cntl_button_patt_capture.EnableWindow(false);
	m_button_start.EnableWindow(false);
	m_button_stop.EnableWindow(false);
	m_cntl_combo_camera_id.EnableWindow(false);
	m_cntl_combo_fps.EnableWindow(false);
	m_cntl_combo_image_size.EnableWindow(false);

	// here, error check is necessary.
	m_pPattCapThread = AfxBeginThread(PattCapThread,this);
}

void CQPServerDlg::OnBnClickedButtonStopPatternCapture()
{
	m_PattCapLoopEnable = false;
}

// PattCapThread (Parent)
UINT CQPServerDlg::PattCapThread(LPVOID pParam)
{
	((CQPServerDlg*)pParam)->PattCapThread();
	return 0;
}


int CQPServerDlg::RegisterNewPattern(IplImage *imgIn)
{
	// convert
	IplImage *imgARTK = cvCreateImage(cvGetSize(imgIn),IPL_DEPTH_8U,4);
	if ( imgARTK == NULL ) return -1;
	cvCvtColor(imgIn,imgARTK,CV_BGR2BGRA);

	// detection
	int marker_num;
	ARMarkerInfo *marker_info;
	arDetectMarker( (ARUint8*)(imgARTK->imageData), 100, &marker_info, &marker_num );
	if ( marker_num <= 0 ) return -1;

	int id = m_cntl_list_pattern.GetItemCount();
	char name[256];
	char fname_patt[256], save_patt[256];
	char fname_clip[256], save_clip[512];

	sprintf_s( name, sizeof(name), "P%03d", id );
	sprintf_s( fname_patt,sizeof(fname_patt),"%03d.patt",id);
	sprintf_s( save_patt, sizeof(save_patt), "Data/%s",fname_patt);
	sprintf_s( fname_clip,sizeof(fname_clip),"%03d.jpg",id);
	sprintf_s( save_clip,sizeof(save_clip),"Data/%s",fname_clip);

	IplImage* imgClip = cvCreateImage( cvSize(80,80),IPL_DEPTH_8U, 4 );
	ClipImage( imgARTK, imgClip, &marker_info[0] );
	cvuSafeSaveImage( save_clip, imgClip );
	cvReleaseImage(&imgClip);

	arSavePatt( (ARUint8*)(imgARTK->imageData),&marker_info[0], save_patt );
	arLoadPatt( save_patt );
	cvReleaseImage(&imgARTK);

	PATTERN_DATA patt;
	patt.enable = true;
	patt.name   = name;
	patt.filename = fname_patt;
	patt.id		= id;
	patt.width	= m_default_patt_width;

	m_pattern_list.push_back(patt);
	InsertItemToList( patt );
	SavePatternList();

	return 0;
}



// PattCapThread (Main)
void CQPServerDlg::PattCapThread()
{
	TRACE("Pattern Capture Thread : start\n");
	m_PattCapLoopEnable = true;
	arDebug = false;

	// Window Setup --------------------------------------------------------------------------
	char *win = "Pattern Capture";
	cvNamedWindow(win);
	cvSetMouseCallback(win,on_mouse);

	// Image for ARToolKit Setup -------------------------------------------------------------
	int width = m_capture.QueryFrame()->width;
	int height = m_capture.QueryFrame()->height;

	IplImage* imgARTK = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U, 4);
	IplImage* imgFrame = NULL;	// do not release

	// Camera parameter setup ----------------------------------------------------------------
	ARParam cparam;
	ARParam wparam;
	arParamLoad(PATH_CAMERA_PARAM_FILE, 1,&wparam);
	arParamChangeSize(&wparam,width,height,&cparam);
	arParamDisp(&cparam);
	arInitCparam(&cparam);

	ARMarkerInfo *marker_info;
	int marker_num;
	bool prev_mouseL = false;

	// Image Processing Loop
	while (m_PattCapLoopEnable) {
		imgFrame = m_capture.QueryFrame();

		cvCvtColor(imgFrame,imgARTK,CV_BGR2BGRA);
		arDetectMarker( (ARUint8*)(imgARTK->imageData), 100, &marker_info, &marker_num );

		int k=-1;
		for (int i=0; i<marker_num; i++) {
			if ( InBoundingBox( &marker_info[i], g_mouse.x, g_mouse.y ) ){
				k = i;
			}
		}
		if ( g_mouse.L && !prev_mouseL && k!=-1 ) {
			RegisterNewPattern(imgFrame);
		}
		prev_mouseL = g_mouse.L;

		// visualization
		if ( arDebug && arImage != NULL ) {
			memcpy( imgARTK->imageData, arImage, imgARTK->imageSize );
		}
		for (int i=0; i<marker_num; i++) {
			if (k==i) {
				DrawMarkerLine2( imgARTK, &marker_info[i], CV_RGB(255,0,0));
			} else {
				DrawMarkerLine2( imgARTK, &marker_info[i], CV_RGB(255,255,0));
			}
		}	
		cvShowImage(win,imgARTK);
		int key = cvWaitKey(1);
		if (key==0x1B) break;	// ESC
		if (key=='d') arDebug = !arDebug;

		// should be after cvWaitKey()
		void* win_is_active = cvGetWindowHandle(win);
		if ( !win_is_active ) break;
	}

	// Release image processing element
	cvDestroyWindow(win);
	cvReleaseImage(&imgARTK);
	m_capture.Release();	// Is this working??

	m_cntl_button_patt_capture.EnableWindow(true);
	m_button_start.EnableWindow(true);
	m_button_stop.EnableWindow(true);
	m_cntl_combo_camera_id.EnableWindow(true);
	m_cntl_combo_fps.EnableWindow(true);
	m_cntl_combo_image_size.EnableWindow(true);

	m_pPattCapThread = NULL;

	TRACE("Pattern Capture Thread : end\n");
}

// Thread (Parent)
UINT CQPServerDlg::Thread(LPVOID pParam)
{
	((CQPServerDlg*)pParam)->Thread();
	return 0;
}

// Thread (Main)
void CQPServerDlg::Thread()
{
	TRACE("Main Thread : start\n");
	arDebug = false;

	m_loopEnable = true;
	EnableServer();

	// Window Setup --------------------------------------------------------------------------
	BOOL use_win = false;
	BOOL show_preview = false;
	char *win = "Preview";

	// Image for ARToolKit Setup -------------------------------------------------------------
	int width = m_capture.QueryFrame()->width;
	int height = m_capture.QueryFrame()->height;
	IplImage* imgARTK = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U, 4);

	// Pattern Setup -------------------------------------------------------------------------
	RegisterPattern( m_pattern_list );

	// Camera parameter setup ----------------------------------------------------------------
	ARParam cparam;
	ARParam wparam;
	arParamLoad(PATH_CAMERA_PARAM_FILE,1,&wparam);
	arParamChangeSize(&wparam,width,height,&cparam);
	arParamDisp(&cparam);
	arInitCparam(&cparam);

	// Begin server listen ------------------------------------------------------------------
	m_net.listen(m_edit_port,Server);

	// Image Processing Loop
	while (m_loopEnable) {
		show_preview = m_cntl_check_preview.GetCheck();

		if ( show_preview && !use_win ) {
			cvNamedWindow(win);
			use_win = true;
		}
		if ( !show_preview && use_win ) {
			cvDestroyWindow(win);
			use_win = false;
		}

		for (int i=0; i<m_cntl_list_pattern.GetItemCount(); i++) {
			if (m_cntl_list_pattern.GetCheck(i)) {
				arActivatePatt(i);
			} else {
				arDeactivatePatt(i);
			}
		}

		// Detection
		g_mtMarkerInfo.lock();
			g_imgCapture = m_capture.QueryFrame();
			Detection( g_imgCapture, imgARTK, 100, m_pattern_list, g_markerInfo );
			if ( show_preview ) {
				if ( arDebug && arImage != NULL ) {
					memcpy( imgARTK->imageData, arImage, imgARTK->imageSize );
				}
				for (unsigned int j=0; j<g_markerInfo.size(); j++) {
					DrawMarkerInfo( imgARTK, g_markerInfo[j] );
				}
			}
		g_mtMarkerInfo.unlock();

		// Preview
		if ( show_preview ) {
			cvShowImage(win,imgARTK);
			int key = cvWaitKey(1);
			if (key=='d') arDebug = !arDebug;
			void* win_is_active = cvGetWindowHandle(win);
			if (!win_is_active) {
				m_cntl_check_preview.SetCheck(false);
			}
		}

		// Output
		g_mtMarkerInfo.lock();
			CString output;
			char all[2048];
			char ibuf[1024];
			sprintf_s( all, sizeof(all), "Num= %d\r\n", g_markerInfo.size() );
			for ( unsigned int j=0; j<g_markerInfo.size(); j++) {
				MakeMarkerInfoString( g_markerInfo[j], g_enable_list, ibuf, sizeof(ibuf) );
				strcat_s( all, sizeof(all), ibuf );
				strcat_s( all, sizeof(all), "\r\n" );
			}
			output = all;
			m_cntl_edit_output.SetWindowTextW(output);
		g_mtMarkerInfo.unlock();


		// Register
		if ( g_regist_patt ) {
			IplImage *imgFile = cvuSafeLoadImage(g_regist_file.c_str());
			if ( imgFile != NULL ) {
				RegisterNewPattern( imgFile );
				cvReleaseImage(&imgFile);
			}
			g_regist_patt = false;
		}
	}

	// Release image processing element
	if ( use_win ) cvDestroyWindow(win);
	cvReleaseImage(&imgARTK);
	m_capture.Release();	// Is this working??

	// Release pattern info
	for (unsigned int i=0; i<m_pattern_list.size(); i++) {
		arFreePatt(i);
	}
	DisableServer();
	m_net.stop_listen();
	UnlockInterface();

	m_pThread = NULL;
	TRACE("Main Thread : end\n");
}


int CQPServerDlg::MakeDataFormat(void)
{
	UpdateData(true);

	g_enable_list.name  = m_check_name;
	g_enable_list.pos2d = m_check_pos2d;
	g_enable_list.dvec  = m_check_dvec;
	g_enable_list.pos3d = m_check_pos3d;
	g_enable_list.glmatrix = m_check_glmatrix;
	g_enable_list.cf       = m_check_cf;
	g_enable_list.area     = m_check_area;
	g_enable_list.vertex   = m_check_vertex;

	char format[512];
	MakeDataFormatString(g_enable_list,format,sizeof(format));
	char all[512];
	sprintf_s( all, sizeof(all), "Num= <count>\r\n%s",format);
	m_edit_data_format = all;

	UpdateData(false);
	return 0;
}

int CQPServerDlg::InitList(void)
{
    LVCOLUMN    lvc;
    int         i;
	TCHAR       caption[][32] = {_T("Name"), _T("Width [mm]") };
    const int   clmNum = sizeof caption / sizeof caption[0];
    int         err = 0;

	m_cntl_list_pattern.SetExtendedStyle(LVS_EX_FULLROWSELECT);
	m_cntl_list_pattern.SetExtendedStyle((m_cntl_list_pattern.GetExtendedStyle()|LVS_EX_CHECKBOXES));

    lvc.mask = LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
    for (i = 0; i < clmNum; i++) {
        lvc.iSubItem    = i;
        lvc.pszText     = caption[i];
        lvc.cx          = 100;
        if (m_cntl_list_pattern.InsertColumn(i, &lvc) == -1) {err = 1; break;}
    }
    
    return err;
}

int CQPServerDlg::InsertItemToList( PATTERN_DATA pattData )
{
	LVITEM       lvi;
	CString      str;
	int          index = 0;
	int          err = 0;

    lvi.mask = LVIF_TEXT;

	// Name
    if (!err)
    {
		str = pattData.name.c_str();
        lvi.iItem = m_cntl_list_pattern.GetItemCount();
		lvi.iSubItem = 0;
        lvi.pszText = const_cast<LPTSTR>(static_cast<LPCTSTR>(str));
        if ((index = m_cntl_list_pattern.InsertItem(&lvi)) == -1) err = 1;
    }

	// Check
	if (pattData.enable) {
		m_cntl_list_pattern.SetCheck(index);
	}

	// Width
    if (!err)
    {
        str.Format(_T("%.1lf"), pattData.width);
        lvi.iItem = index;
        lvi.iSubItem = 1;
        lvi.pszText = const_cast<LPTSTR>(static_cast<LPCTSTR>(str));
        if (!m_cntl_list_pattern.SetItem(&lvi)) err = 1;
    }

	// focus
	m_cntl_list_pattern.SetItemState( index, LVIS_SELECTED|LVIS_FOCUSED,LVIS_SELECTED|LVIS_FOCUSED );
	return err;
}

void CQPServerDlg::OnNMRClickListPattern(NMHDR *pNMHDR, LRESULT *pResult)
{
	CMenu menu;
	menu.CreatePopupMenu();

	if ( m_pThread ) {
		menu.AppendMenu(MF_STRING|MF_GRAYED,IDM_DELETE_ITEM ,_T("Remove this pattern") );
	} else {
		menu.AppendMenu(MF_STRING|MF_ENABLED,IDM_DELETE_ITEM ,_T("Remove this pattern") );
	}
	POINT point;
	GetCursorPos(&point);
	menu.TrackPopupMenu( TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this );
	menu.DestroyMenu();

	*pResult = 0;
}


int CQPServerDlg::EnableServer(void)
{
	g_mtServerEnable.lock();
		g_serverEnable = true;
	g_mtServerEnable.unlock();

	return 0;
}

int CQPServerDlg::DisableServer(void)
{
	g_mtServerEnable.lock();
		g_serverEnable = false;
	g_mtServerEnable.unlock();

	return 0;
}

// Click on pattern list
void CQPServerDlg::OnLvnItemchangedListPattern(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

	if ((pNMLV->uNewState & LVIS_SELECTED) && !(pNMLV->uOldState & LVIS_SELECTED)) {
		UpdateData(true);

		// Show pattern
		char filename[256];
		sprintf_s(filename,sizeof(filename),"Data/%03d.jpg",pNMLV->iItem);
		m_imgPatt = cvuSafeLoadImage(filename);
		ShowImage( IDC_PICT_PATT, m_imgPatt );

		// Show info on editbox
		m_edit_patt_name = m_cntl_list_pattern.GetItemText( pNMLV->iItem, 0 );
		m_edit_patt_width = _tstof(m_cntl_list_pattern.GetItemText(pNMLV->iItem, 1));
		UpdateData(false);
	}

	*pResult = 0;
}


void CQPServerDlg::DeleteItemFromList(void)
{
	int index = -1;
	while ((index = m_cntl_list_pattern.GetNextItem(index, LVNI_ALL |LVNI_SELECTED)) != -1){

		int old_num = m_cntl_list_pattern.GetItemCount();

		char filename[256];
		sprintf_s( filename, sizeof(filename), "Data/%03d.patt",index );
		remove(filename);
		TRACE("Remove: %s\n",filename);

		sprintf_s( filename, sizeof(filename), "Data/%03d.jpg",index );
		remove(filename);
		TRACE("Remove: %s\n",filename);

		for (int i=index+1; i<old_num; i++) {
			char oldname[256];
			char newname[256];
			sprintf_s( oldname, sizeof(oldname), "Data/%03d.patt",i);
			sprintf_s( newname, sizeof(newname), "Data/%03d.patt",i-1);
			rename( oldname, newname );
			TRACE("Rename: %s->%s\n",oldname,newname);
			sprintf_s( oldname, sizeof(oldname), "Data/%03d.jpg",i);
			sprintf_s( newname, sizeof(newname), "Data/%03d.jpg",i-1);
			rename( oldname, newname );
			TRACE("Rename: %s->%s\n",oldname,newname);
		}
		m_cntl_list_pattern.DeleteItem(index);
		m_cntl_list_pattern.SetItemState( index, LVIS_SELECTED|LVIS_FOCUSED,LVIS_SELECTED|LVIS_FOCUSED );

		if ( m_cntl_list_pattern.GetItemCount()==0 ) {
			m_imgPatt = cvuSafeLoadImage("no_pattern.bmp");
			ShowImage( IDC_PICT_PATT, m_imgPatt );
			m_edit_patt_name = _T("");
			m_edit_patt_width = 0;
			UpdateData(false);
		}
	}
}

void CQPServerDlg::UpdatePatternInfo()
{
	int index = -1;
	while ((index = m_cntl_list_pattern.GetNextItem(index, LVNI_ALL |LVNI_SELECTED)) != -1){
		UpdateData(true);
		m_cntl_list_pattern.SetItemText( index, 0, m_edit_patt_name );
		CString str;
		str.Format(_T("%.1lf"), m_edit_patt_width );
		m_cntl_list_pattern.SetItemText( index, 1, const_cast<LPTSTR>(static_cast<LPCTSTR>(str)) );
		UpdateData(false);
	}
}


void CQPServerDlg::OnBnClickedCheckName()
{
	MakeDataFormat();
	SaveConfigFile();
}
void CQPServerDlg::OnBnClickedCheckPos2d()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckDvec()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckPos3d()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckArea()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckCf()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckVertex()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckGlmatrix()
{
	MakeDataFormat();
	SaveConfigFile();
}

void CQPServerDlg::OnEnChangeEditPort()
{
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckAutoStart()
{
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedCheckPreview()
{
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedRadioDataflow1()
{
	SaveConfigFile();
}

void CQPServerDlg::OnBnClickedRadioDataflow2()
{
	SaveConfigFile();
}

void CQPServerDlg::OnCbnSelchangeComboCameraId()
{
	SaveConfigFile();
}

void CQPServerDlg::OnCbnSelchangeComboFps()
{
	SaveConfigFile();
}

void CQPServerDlg::OnCbnSelchangeComboImageSize()
{
	SaveConfigFile();
}

void CQPServerDlg::OnEnChangeEditPatternName()
{
	UpdatePatternInfo();
	UpdateData(true);
	SavePatternList();
}

void CQPServerDlg::OnEnChangeEditPatternWidth()
{
	UpdatePatternInfo();
	UpdateData(true);
	SavePatternList();
}


// X button
void CQPServerDlg::OnBnClickedCancel()
{
	if ( m_pKillThread !=NULL ) return;
	if ( m_pThread != NULL || m_pPattCapThread != NULL) {
		m_pKillThread = AfxBeginThread(KillThread,this);
		return;
	}
	if ( m_imgPatt!=NULL ) {
		cvReleaseImage(&m_imgPatt);
		m_imgPatt = NULL;
	}

	UpdateData(true);
	SavePatternList();
	OnCancel();
}

// KillThread (Parent)
UINT CQPServerDlg::KillThread(LPVOID pParam)
{
	((CQPServerDlg*)pParam)->KillThread();
	return 0;
}

// KillThread (Main)
void CQPServerDlg::KillThread()
{
	TRACE("Kill Thread : start\n");
	m_loopEnable = false;
	m_PattCapLoopEnable = false;

	while ( m_pThread != NULL || m_pPattCapThread != NULL) {
		Sleep(1);
	}
	m_pKillThread = NULL;
	TRACE("Kill Thread : end\n");
	OnBnClickedCancel();
}
